//キャラクタの入力操作を受け付けるスクリプト(ジョイスティック対応)

//最適化のための定義(負荷の軽減に役立つが、スクリプトの文法チェックが厳しくなる)
#pragma strict

//このスクリプトが必要とするコンポーネント(付加されていない場合は自動で付加される)
@script RequireComponent(PlatformCharacterMotor)


//
//内部で使用する変数
//
//コンポーネントの参照
private var motor : PlatformCharacterMotor; //キャラクタ制御スクリプトコンポーネント
private var shooterController : ShooterController2; //剣発射スクリプトコンポーネント
private var inputLook : InputLook;  //視点変更スクリプト

private var jumpTapcount : int = 2; //ジャンプに必要な最小のタップ数
private var fireTapcount : int = 2; //剣の発射に必要な最小のタップ数

private var canFire : boolean = true;   //剣を発射できるかどうか
private var canJump : boolean = true;   //ジャンプができるかどうか


//
//プロパティ
//
var moveJoystick : Joystick;    //移動に使用するジョイスティック
var jumpJoystick : Joystick;    //ジャンプに使用するジョイスティック
var fireJoystick : Joystick;    //剣の発射に使用するジョイスティック
var lookJoystick : Joystick;    //視点移動に使用するジョイスティック
var runMinRatio : float = 0.5;  //ダッシュと判定するジョイスティックの傾きの度合い


//スクリプトの導入時に一度だけ呼ばれる
//コンポーネントの参照の取得と代入、およびジョイスティックの初期導入
function Awake() {

    //コンポーネントの参照を取得して代入する
	motor = GetComponent(PlatformCharacterMotor) as PlatformCharacterMotor;
	shooterController = GetComponent(ShooterController2) as ShooterController2;
    inputLook = GameObject.FindSceneObjectsOfType(InputLook)[0] as InputLook;
	
	//ジョイスティックの初期導入
	SetupJoystick();
	
}

//ジョイスティックの参照を必要に応じて取得、代入する
private function SetupJoystick(){
    var joysticks: Joystick[] = FindSceneObjectsOfType(Joystick) as Joystick[]; //シーン中のジョイスティックを全て取得

	//ジョイスティックがプロパティで指定されていない場合は、それぞれ自動でセットする
	
	//移動のジョイスティック
	if( !moveJoystick ){
        moveJoystick = FindObjectByName( joysticks, "Left") as Joystick;   //左ジョイスティックを検索して代入
	}
	
	//視点移動のジョイスティック
	if( !lookJoystick ){
        lookJoystick = FindObjectByName( joysticks, "Right") as Joystick;   //左ジョイスティックを検索して代入	    
	}
	
	//ジャンプのジョイスティック
	if( !jumpJoystick ){
        jumpJoystick = FindObjectByName( joysticks, "Right") as Joystick;   //左ジョイスティックを検索して代入	    
	}
	
	//剣発射のジョイスティック
	if( !fireJoystick ){
        fireJoystick = FindObjectByName( joysticks, "Left") as Joystick;   //左ジョイスティックを検索して代入	
	}

}

//オブジェクトの配列から、指定名の一部が含まれるオブジェクトを取得する
private static function FindObjectByName( objects : UnityEngine.Object[], name: String ) : UnityEngine.Object{

    //配列の数だけ名前を比較して検索する
    for( var i: int= 0; i < objects.length; i++ ){
        var object: UnityEngine.Object = objects[i];
        if( object.name.Contains( name ) ){
            return object;  //名前の一部を含んでいるのでこのオブジェクトを返す
        }
    }
    
    //見つからなかった場合はnull(無し)を返す
    return null;
}

//毎フレーム呼ばれる
//入力操作の処理
function Update() {


    //
    //ジャンプの処理
    //

    //ジャンプができるならジャンプする
    if( canJump ){
        var isJump: Boolean = jumpJoystick.tapCount == jumpTapcount;    //指定数のタップがあればジャンプ
	    motor.inputJump = isJump;    //キャラクター制御クラスに、ジャンプ入力の有無を代入する
	    canJump = !isJump;
	}
	else{   
	    //ジャンプ後にジョイスティックから指を離さないと再びジャンプはできない
	    motor.inputJump = false;
	    if( jumpJoystick.IsFingerDown() == false ){
	        canJump = true;
	    }
	}
	
	//
	//剣発射の処理
	//
	
	//発射ができるなら発射する
	if( canFire ){
	    var isFire: boolean = fireJoystick.tapCount == fireTapcount;    //指定数のタップがあれば発射
    	shooterController.inputFire = isFire;  //剣発射スクリプトコンポーネントに発射の有無を代入
	    canFire = !isFire;
    }
    else{
        //発射後にジョイスティックから指を離さないと再び発射はできない
	    shooterController.inputFire = false;    
        if( fireJoystick.IsFingerDown() == false ){
            canFire = true;
        }
	}

    
    //
    //移動の処理
    //
	var moveJoystickPosition:Vector2 = moveJoystick.position;   //ジョイスティックの位置を取得する
	var directionVector : Vector3 = new Vector3(moveJoystickPosition.x, 0, moveJoystickPosition.y); //ジョイスティックの横と縦の座標を、三次元ベクトルのX,Z(横と奥)に置換する。
	motor.inputMove = directionVector;  //キャラクター制御クラスに、ジョイスティックの傾き(方向)を代入する

    //
    //ダッシュの処理
    //
	var directionRatio: float = directionVector.sqrMagnitude;   //ジョイスティックの傾きの度合いを算出(最大で1)
	var runRatio: float  = Mathf.Max( 0, directionRatio - runMinRatio); //ダッシュの度合いを算出
	var inputRun: float = runRatio / (1 - runMinRatio );    //ダッシュの度合いを、1を最大とする比重値に変換する
    motor.inputRun = inputRun;  //キャラクター制御スクリプトのダッシュ変数に代入
    
    //
    //視点の処理
    //視点変更用のジョイスティックから位置を取得して、視点操作スクリプト(InputLook)に代入する
    //
    var lookPosition: Vector2 = lookJoystick.position;
    inputLook.axisX = lookPosition.x;
    inputLook.axisY = lookPosition.y;
    
    
    
}
